@@ -1,26 +1,27 @@
 package Apache::Session::Browseable::LDAP;
 
 use strict;
-use vars qw(@ISA $VERSION);
-
-$VERSION = '0.8';
-@ISA     = qw(Apache::Session);
 
 use Apache::Session;
 use Apache::Session::Lock::Null;
 use Apache::Session::Browseable::Store::LDAP;
-use Apache::Session::Generate::MD5;
-use Apache::Session::Serialize::Base64;
+use Apache::Session::Generate::SHA256;
+use Apache::Session::Serialize::JSON;
+use Apache::Session::Browseable::_common;
+use Net::LDAP::Util qw(escape_filter_value);
+
+our $VERSION = '1.2.2';
+our @ISA     = qw(Apache::Session Apache::Session::Browseable::_common);
 
 sub populate {
     my $self = shift;
 
     $self->{object_store} = new Apache::Session::Browseable::Store::LDAP $self;
     $self->{lock_manager} = new Apache::Session::Lock::Null $self;
-    $self->{generate}     = \&Apache::Session::Generate::MD5::generate;
-    $self->{validate}     = \&Apache::Session::Generate::MD5::validate;
-    $self->{serialize}    = \&Apache::Session::Serialize::Base64::serialize;
-    $self->{unserialize}  = \&Apache::Session::Serialize::Base64::unserialize;
+    $self->{generate}     = \&Apache::Session::Generate::SHA256::generate;
+    $self->{validate}     = \&Apache::Session::Generate::SHA256::validate;
+    $self->{serialize}    = \&Apache::Session::Serialize::JSON::serialize;
+    $self->{unserialize}  = \&Apache::Session::Serialize::JSON::unserialize;
 
     return $self;
 }
@@ -28,62 +29,79 @@ sub populate {
 sub unserialize {
     my $session = shift;
     my $tmp = { serialized => $session };
-    Apache::Session::Serialize::Base64::unserialize($tmp);
+    Apache::Session::Serialize::JSON::unserialize($tmp);
     return $tmp->{data};
 }
 
 sub searchOn {
     my ( $class, $args, $selectField, $value, @fields ) = @_;
 
-    my %res = ();
     my $index =
       ref( $args->{Index} ) ? $args->{Index} : [ split /\s+/, $args->{Index} ];
     if ( grep { $_ eq $selectField } @$index ) {
-        my $ldap =
-          Apache::Session::Browseable::Store::LDAP::ldap( { args => $args } );
-        my $msg = $ldap->search(
-            base => $args->{ldapConfBase},
-            filter =>
-              "(&(objectClass=applicationProcess)(ou=${selectField}_$value))",
-
-            #scope => 'base',
-            attrs => [ 'description', 'cn' ],
-        );
-        if ( $msg->code ) {
-            Apache::Session::Browseable::Store::LDAP->logError($msg);
-        }
-        else {
-            foreach my $entry ( $msg->entries ) {
-                my $id = $entry->get_value('cn') or die;
-                my $tmp = $entry->get_value('description');
-                next unless ($tmp);
-                eval { $tmp = unserialize($tmp); };
-                next if ($@);
-                if (@fields) {
-                    $res{$id}->{$_} = $tmp->{$_} foreach (@fields);
-                }
-                else {
-                    $res{$id} = $tmp;
-                }
-            }
-        }
+        ( $selectField, $value ) = escape_filter_value( $selectField, $value );
+        return $class->_query( $args, $selectField, $value, @fields );
     }
     else {
-        $class->get_key_from_all_sessions(
-            $args,
-            sub {
-                my $entry = shift;
-                my $id    = shift;
-                return undef unless ( $entry->{$selectField} eq $value );
-                if (@fields) {
-                    $res{$id}->{$_} = $entry->{$_} foreach (@fields);
-                }
-                else {
-                    $res{$id} = $entry;
-                }
-                undef;
+        return $class->SUPER::searchOn( $args, $selectField, $value, @fields );
+    }
+}
+
+sub searchOnExpr {
+    my ( $class, $args, $selectField, $value, @fields ) = @_;
+
+    my $index =
+      ref( $args->{Index} ) ? $args->{Index} : [ split /\s+/, $args->{Index} ];
+    if ( grep { $_ eq $selectField } @$index ) {
+        ( $selectField, $value ) = escape_filter_value( $selectField, $value );
+        $value =~ s/\\2a/\*/gi;
+        return $class->_query( $args, $selectField, $value, @fields );
+    }
+    else {
+        return $class->SUPER::searchOn( $args, $selectField, $value, @fields );
+    }
+}
+
+sub _query {
+    my ( $class, $args, $selectField, $value, @fields ) = @_;
+    $args->{ldapObjectClass}      ||= 'applicationProcess';
+    $args->{ldapAttributeId}      ||= 'cn';
+    $args->{ldapAttributeContent} ||= 'description';
+    $args->{ldapAttributeIndex}   ||= 'ou';
+
+    my %res = ();
+    my $ldap =
+      Apache::Session::Browseable::Store::LDAP::ldap( { args => $args } );
+    my $msg = $ldap->search(
+        base   => $args->{ldapConfBase},
+        filter => "(&(objectClass="
+          . $args->{ldapObjectClass} . ")("
+          . $args->{ldapAttributeIndex}
+          . "=${selectField}_$value))",
+
+        #scope => 'base',
+        attrs => [ $args->{ldapAttributeContent}, $args->{ldapAttributeId} ],
+    );
+
+    $ldap->unbind();
+
+    if ( $msg->code ) {
+        Apache::Session::Browseable::Store::LDAP->logError($msg);
+    }
+    else {
+        foreach my $entry ( $msg->entries ) {
+            my $id = $entry->get_value( $args->{ldapAttributeId} ) or die;
+            my $tmp = $entry->get_value( $args->{ldapAttributeContent} );
+            next unless ($tmp);
+            eval { $tmp = unserialize($tmp); };
+            next if ($@);
+            if (@fields) {
+                $res{$id}->{$_} = $tmp->{$_} foreach (@fields);
             }
-        );
+            else {
+                $res{$id} = $tmp;
+            }
+        }
     }
     return \%res;
 }
@@ -92,6 +110,11 @@ sub get_key_from_all_sessions {
     my $class = shift;
     my $args  = shift;
     my $data  = shift;
+    $args->{ldapObjectClass}      ||= 'applicationProcess';
+    $args->{ldapAttributeId}      ||= 'cn';
+    $args->{ldapAttributeContent} ||= 'description';
+    $args->{ldapAttributeIndex}   ||= 'ou';
+
     my %res;
 
     my $ldap =
@@ -101,16 +124,22 @@ sub get_key_from_all_sessions {
 
      # VERY STRANGE BUG ! With this filter, description isn't base64 encoded !!!
      #filter => '(objectClass=applicationProcess)',
-        filter => '(&(objectClass=applicationProcess)(ou=*))',
-        attrs  => [ 'cn', 'description' ],
+
+        filter => '(&(objectClass='
+          . $args->{ldapObjectClass} . ')('
+          . $args->{ldapAttributeIndex} . '=*))',
+        attrs => [ $args->{ldapAttributeId}, $args->{ldapAttributeContent} ],
     );
+
+    $ldap->unbind();
+
     if ( $msg->code ) {
         Apache::Session::Browseable::Store::LDAP->logError($msg);
     }
     else {
         foreach my $entry ( $msg->entries ) {
-            my $id = $entry->get_value('cn') or die;
-            my $tmp = $entry->get_value('description');
+            my $id = $entry->get_value( $args->{ldapAttributeId} ) or die;
+            my $tmp = $entry->get_value( $args->{ldapAttributeContent} );
             next unless ($tmp);
             eval { $tmp = unserialize($tmp); };
             next if ($@);
@@ -142,11 +171,15 @@ Apache::Session::Browseable::LDAP - An implementation of Apache::Session::LDAP
 
   use Apache::Session::Browseable::LDAP;
   tie %hash, 'Apache::Session::Browseable::LDAP', $id, {
-    ldapServer       => 'ldap://localhost:389',
-    ldapConfBase     => 'dmdName=applications,dc=example,dc=com',
-    ldapBindDN       => 'cn=admin,dc=example,dc=com',
-    ldapBindPassword => 'pass',
-    Index            => 'uid ipAddr',
+    ldapServer           => 'ldap://localhost:389',
+    ldapConfBase         => 'dmdName=applications,dc=example,dc=com',
+    ldapBindDN           => 'cn=admin,dc=example,dc=com',
+    ldapBindPassword     => 'pass',
+    Index                => 'uid ipAddr',
+    ldapObjectClass      => 'applicationProcess',
+    ldapAttributeId      => 'cn',
+    ldapAttributeContent => 'description',
+    ldapAttributeIndex   => 'ou',
   };
 
 =head1 DESCRIPTION
@@ -160,7 +193,10 @@ Xavier Guimard, E<lt>x.guimard@free.frE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
+=encoding utf8
+
 Copyright (C) 2009 by Xavier Guimard
+Copyright (C) 2013, 2015 by Clement Oudot
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.10.0 or,
